"""
Constants & functions for defining PCAPNG options.

See:

http://xml2rfc.tools.ietf.org/cgi-bin/xml2rfc.cgi?url=https://raw.githubusercontent.com/pcapng/pcapng/master/draft-tuexen-opsawg-pcapng.xml&modeAsFormat=html/ascii&type=ascii#formatopt
"""

OPT_END_OF_OPT    =     0

OPT_COMMENT       =     1

#todo need to do validation on data values & lengths
# custom options
OPT_CUSTOM_0      =  2988
OPT_CUSTOM_1      =  2989
OPT_CUSTOM_2      = 19372
OPT_CUSTOM_3      = 19373

#todo need to do validation on data values & lengths
# section header block options
OPT_SHB_HARDWARE  = 2
OPT_SHB_OS        = 3
OPT_SHB_USERAPPL  = 4

#todo need to do validation on data values & lengths
# interface description block options
IF_NAME           =   2
IF_DESCRIPTION    =   3
IF_IPV4ADDR       =   4
IF_IPV6ADDR       =   5
IF_MACADDR        =   6
IF_EUIADDR        =   7
IF_SPEED          =   8
IF_TSRESOL        =   9
IF_TZONE          =  10
IF_FILTER         =  11
IF_OS             =  12
IF_FCSLEN         =  13
IF_TSOFFSET       =  14

def assert_custom_option(opt_code):
    valid_opts = { OPT_CUSTOM_0, OPT_CUSTOM_1, OPT_CUSTOM_2, OPT_CUSTOM_3 }
    assert (opt_code in valid_opts)

def assert_shb_option(opt_code):
    valid_opts = { OPT_SHB_HARDWARE, OPT_SHB_OS, OPT_SHB_USERAPPL }
    assert (opt_code in valid_opts)

def assert_if_option(opt_code):
    valid_opts = {  IF_NAME, IF_DESCRIPTION, IF_IPV4ADDR, IF_IPV6ADDR, IF_MACADDR, IF_EUIADDR,
                    IF_SPEED, IF_TSRESOL, IF_TZONE, IF_FILTER, IF_OS, IF_FCSLEN, IF_TSOFFSET }
    assert (opt_code in valid_opts)



import struct
import sys
import time
import math


def is_python2():
    (major, minor, micro, release_level, serial) = sys.version_info
    return ((major == 2) and (minor == 7))

def is_python3():
    (major, minor, micro, release_level, serial) = sys.version_info
    return ((major == 3) and (minor >= 5))

def assert_python2():
    assert is_python2()


def assert_type_bytearray( arg ):
    assert type( arg ) == bytearray

def assert_type_bytes( arg ):
    assert type( arg ) == bytes

def assert_type_str( arg ):
    assert type( arg ) == str

def assert_type_list( arg ):
    assert type( arg ) == list

def assert_type_dict( arg ):
    assert type( arg ) == dict

def assert_uint8(arg):        # unsigned byte
    assert (0 <= arg <= 255)

def assert_int8(arg):          # signed byte
    assert (-128 <= arg <= 127)


def to_bytes( arg ):
    """Converts arg to a 'bytes' object."""
    return bytes( bytearray( arg ))    # if python2, 'bytes' is synonym for 'str'

def str_to_bytes( arg ):
    """Converts a string arg to a 'bytes' object."""
    assert_type_str( arg )
    """Convert an ASCII string to 'bytes'. Works on both Python2 and Python3."""
    return to_bytes( map(ord,arg))


#todo move to pcap
def fmt_pcap_hdr( ts_sec, ts_usec, incl_len, orig_len ):
    """Format a PCAP block header."""
    packed = struct.pack( '>LLLL', ts_sec, ts_usec, incl_len, orig_len)
    return packed


def split_float( fval ):
    """Splits a float into integer and fractional parts."""
    frac, whole = math.modf( fval )
    micros = int( round( frac * 1000000 ))
    return int(whole), micros

def curr_utc_time_tuple():
    """Returns the current UTC time as a (secs, usecs) tuple."""
    utc_secs = time.time()
    secs, usecs = split_float( utc_secs )
    return secs, usecs

def timetup_to_float( secs, usecs ):
    """Converts a time tuple from (secs, usecs) to float."""
    return secs + (usecs / 1000000.0)

def timetup_subtract( ts1, ts2 ):
    """Subtracts two time tuples in (secs, usecs) format, returning a float result."""
    (s1, us1) = ts1
    (s2, us2) = ts2
    t1 = timetup_to_float( s1, us1 )
    t2 = timetup_to_float( s2, us2 )
    delta = t2 - t1
    return delta

def ChrList_to_str(arg):
    """ Convert a list of characters to a string"""
    #todo verify input type & values [0..255]
    strval = ''.join( arg )
    return strval

#todo move to pcapng.list (or delete?); only(), second(), last(), butlast(), rest()
def first( lst ):
    """Returns the first item in a sequence."""
    return lst[0]

#todo move to pcapng.bytes
#-----------------------------------------------------------------------------

def block32_ceil_bytes(curr_len):
    """Returns the number of bytes (n >= curr_len) at the next 32-bit boundary"""
    curr_blks = float(curr_len) / 4.0
    pad_blks = int( math.ceil( curr_blks ))
    pad_len = pad_blks * 4
    return pad_len

def pad_bytes(data, tgt_length, padval=0):
    """Add (n>=0) 'padval' bytes to extend data to tgt_length"""
    elem_needed = tgt_length - len(data)
    assert (elem_needed >= 0), "padding cannot be negative"
    result = to_bytes(data) + to_bytes( [padval] )*elem_needed
    return result

def block32_pad_bytes(data):
    """Pad data with (n>=0) 0x00 bytes to reach the next 32-bit boundary"""
    pad_len = block32_ceil_bytes(len(data))
    result = pad_bytes(data, pad_len)
    return result

def assert_block32_length(data):
    """Assert that data length is at a 32-bit boundary"""
    assert (0 == len(data) % 4), "data must be 32-bit aligned"
    return True




import struct
import pcapng.linktype
import pcapng.core
import pcapng.option
import pcapng.util
from pcapng.util import to_bytes, str_to_bytes


def test_option_endofopt():
    assert (0,0) == struct.unpack( '=HH', pcapng.core.option_endofopt())

def test_option_codec():
    def assert_option_codec(opt_code, opt_value):
        (res_code, res_bytes) = pcapng.core.option_decode(
            pcapng.core.option_encode(opt_code, opt_value))
        assert res_code   == opt_code
        assert res_bytes  == to_bytes(opt_value)

    assert_option_codec( 0, [] )
    assert_option_codec( 1, [1,] )
    assert_option_codec( 2, [1,2, ] )
    assert_option_codec( 3, [1,2,3,] )
    assert_option_codec( 4, [1,2,3,4,] )
    assert_option_codec( 5, [1,2,3,4,5] )

def test_options_codec():
    def assert_options_codec( opts_dict ):
        opts_dict_result = pcapng.core.options_decode(
                           pcapng.core.options_encode( opts_dict ))
        assert opts_dict_result == opts_dict

    val0 = str_to_bytes( '' )
    val1 = str_to_bytes( 'a' )
    val2 = str_to_bytes( 'Doh!' )
    assert_options_codec(  { 0:val0 } )
    assert_options_codec(  { 0:val0,
                             1:val1 } )
    assert_options_codec(  { 0:val0,
                             1:val1,
                             2:val2 } )

def test_option_comment_codec():
    def assert_comment_codec( str_val ):
        result = pcapng.core.option_comment_decode(
                 pcapng.core.option_comment_encode(str_val))
        assert result == str_val

    assert_comment_codec( '' )
    assert_comment_codec( 'a' )
    assert_comment_codec( 'go' )
    assert_comment_codec( 'ray' )
    assert_comment_codec( 'Doh!' )
    assert_comment_codec( 'How do you like me now?' )


def test_section_header_block():
    opts = { pcapng.option.OPT_SHB_HARDWARE  : "Dell",
             pcapng.option.OPT_SHB_OS        : "Ubuntu",
             pcapng.option.OPT_SHB_USERAPPL  : "IntelliJ Idea" }
    blk_str     = pcapng.core.section_header_block_encode( opts )
    blk_data    = pcapng.core.section_header_block_decode( blk_str )
    pcapng.util.assert_type_str( blk_str )
    pcapng.util.assert_type_dict( blk_data )
    assert blk_data['block_type']           == 0x0A0D0D0A
    assert blk_data['block_total_len']      == len( blk_str )
    assert blk_data['block_total_len']      == blk_data['block_total_len_end']
    assert blk_data['byte_order_magic']     == 0x1A2B3C4D
    assert blk_data['major_version']        == 1
    assert blk_data['minor_version']        == 0
    assert blk_data['section_len']          == -1
    assert blk_data['options_dict']         == opts

def test_interface_desc_block():
    opts = { pcapng.option.IF_NAME           : "SuperSpeed",
             pcapng.option.IF_DESCRIPTION    : "don't you wish",
             pcapng.option.IF_IPV4ADDR       : to_bytes( [ 192,168,13,7,  255,255,255,0 ] ),
             pcapng.option.IF_OS             : "NitrOS" }
    blk_str    = pcapng.core.interface_desc_block_encode( opts )
    blk_data   = pcapng.core.interface_desc_block_decode(blk_str)
    pcapng.util.assert_type_str( blk_str )
    pcapng.util.assert_type_dict( blk_data )
    assert blk_data['block_type']          == 0x00000001
    assert blk_data['block_total_len']     == len(blk_str)
    assert blk_data['block_total_len']     == blk_data['block_total_len_end']
    assert blk_data['link_type']           == pcapng.linktype.LINKTYPE_ETHERNET
    assert blk_data['reserved']            == 0
    assert blk_data['snaplen']             == 0
    assert blk_data['options_dict']        == opts

def test_simple_pkt_block():
    blk_str   = pcapng.core.simple_pkt_block_encode('abc')
    blk_data  = pcapng.core.simple_pkt_block_decode(blk_str)
    pcapng.util.assert_type_str( blk_str )
    pcapng.util.assert_type_dict( blk_data )
    assert blk_data['block_type']           == 0x00000003
    assert blk_data['block_tot_len']        == 20
    assert blk_data['block_tot_len']        == blk_data['block_tot_len_end']
    assert blk_data['block_tot_len']        == len(blk_str)
    assert blk_data['block_tot_len']        == 16 + blk_data['pkt_data_pad_len']
    assert blk_data['original_pkt_len']     == 3
    assert blk_data['pkt_data']             == 'abc'




#todo add brocade copyright / license
import struct
import pcapng.linktype
import pcapng.option
import pcapng.util
from pcapng.util import to_bytes

#todo think about how to handle a block of packets
#todo look at "docopt" usage -> cmdopts processing

#-----------------------------------------------------------------------------
pcapng.util.assert_python2()    #todo make work for python 2.7 or 3.3 ?
#-----------------------------------------------------------------------------


def option_endofopt():
    """Returns a bytes block for 'end of options' """
    result = struct.pack( '=HH', pcapng.option.OPT_END_OF_OPT, 0 )
    return result

# #todo add all options ability

def option_encode(opt_code, opt_bytes):
    """Encodes an option into a bytes block."""
    data_len_orig   = len(opt_bytes)
    data_pad        = pcapng.util.block32_pad_bytes( to_bytes(opt_bytes))
    result_hdr      = struct.pack( '=HH', opt_code, data_len_orig )
    result          = result_hdr + data_pad
    return result

def options_encode(opts_dict):
    """Encodes an options from a dictionary into a bytes block."""
    pcapng.util.assert_type_dict(opts_dict)
    cum_result = ""
    for opt_code in opts_dict.keys():
        opt_value = opts_dict[ opt_code]
        opt_bytes = option_encode( opt_code, opt_value )
        cum_result += opt_bytes
    return cum_result

def option_decode_rolling(opts_bytes):
    """Given an bytes block of options, decodes and returns the first option and the remaining bytes."""
    pcapng.util.assert_type_bytes(opts_bytes)
    assert 4 <= len(opts_bytes)
    (opt_code, data_len_orig) = struct.unpack( '=HH', opts_bytes[0:4])
    data_len_pad = pcapng.util.block32_ceil_bytes( data_len_orig )
    first_block_len_pad = 4+data_len_pad
    assert first_block_len_pad <= len(opts_bytes)
    first_opt_bytes = opts_bytes[:first_block_len_pad]
    opts_bytes_remaining = opts_bytes[first_block_len_pad:]
    first_opt_value = first_opt_bytes[ 4 : 4+data_len_orig ]
    return ( opt_code, first_opt_value, opts_bytes_remaining )

def options_decode(opts_bytes):
    """Given an bytes block of options, decodes and returns options as a dictionary."""
    pcapng.util.assert_type_bytes(opts_bytes)
    pcapng.util.assert_block32_length(opts_bytes)
    cum_result_dict = {}
    while (0 < len(opts_bytes)):
        ( opt_code, opt_value, opts_bytes_remaining ) = option_decode_rolling(opts_bytes)
        cum_result_dict[ opt_code ] = opt_value
        opts_bytes = opts_bytes_remaining
    return cum_result_dict

def option_decode( block ):
    """Given an bytes block of for one option, decodes and returns the option as a dictionary."""
    opts_dict_result = options_decode( block )
    assert 1 == len( opts_dict_result )
    opt_code  = opts_dict_result.keys()[0]
    opt_bytes = opts_dict_result[opt_code]
    return (opt_code, opt_bytes)

def option_comment_encode( comment_str ):  #todo add unicode => utf-8 support
    "Encodes a string into a comment option."
    pcapng.util.assert_type_str( comment_str )
    result = option_encode( pcapng.option.OPT_COMMENT, comment_str )
    return result

def option_comment_decode(opt_bytes):  #todo add unicode => utf-8 support
    "Decodes a comment option into a string."
    pcapng.util.assert_type_bytes(opt_bytes)
    ( opt_code, opt_bytes ) = option_decode(opt_bytes)
    assert opt_code == pcapng.option.OPT_COMMENT
    return opt_bytes

def section_header_block_encode(opts_dict={}):    #todo data_len
    """Encodes a section header block, including the specified options."""
    block_type = 0x0A0D0D0A
    byte_order_magic = 0x1A2B3C4D
    major_version = 1
    minor_version = 0
    section_len = -1        #todo set to actual (incl padding)

    for opt_code in opts_dict.keys():
        pcapng.option.assert_shb_option(opt_code)
    options_bytes = options_encode(opts_dict)

    block_total_len =    ( 4 +      # block type
                           4 +      # block total length
                           4 +      # byte order magic
                           2 + 2 +  # major version + minor version
                           8 +      # section length
                           len(options_bytes) +
                           4 )      # block total length
    block = ( struct.pack( '=LlLhhq', block_type, block_total_len, byte_order_magic,
                                      major_version, minor_version, section_len )
            + options_bytes
            + struct.pack( '=l', block_total_len ))
    return block

def section_header_block_decode(block):
    """Decodes a bytes block into a section header block, returning a dictionary."""
    assert type( block ) == str
    block_type          = pcapng.util.first( struct.unpack( '=l', block[0:4]   ))
    block_total_len     = pcapng.util.first( struct.unpack( '=l', block[4:8]   ))
    byte_order_magic    = pcapng.util.first( struct.unpack( '=L', block[8:12]  ))
    major_version       = pcapng.util.first( struct.unpack( '=h', block[12:14] ))
    minor_version       = pcapng.util.first( struct.unpack( '=h', block[14:16] ))
    section_len         = pcapng.util.first( struct.unpack( '=q', block[16:24] ))

    options_bytes       = block[24:-4]
    options_dict        = options_decode( options_bytes )

    block_total_len_end = struct.unpack( '=l', block[-4:] )[0]

    block_data = {  'block_type'          : block_type ,
                    'block_total_len'     : block_total_len ,
                    'byte_order_magic'    : byte_order_magic ,
                    'major_version'       : major_version ,
                    'minor_version'       : minor_version ,
                    'section_len'         : section_len ,
                    'options_dict'        : options_dict,
                    'block_total_len_end' : block_total_len_end }
    return block_data

def interface_desc_block_encode( opts_dict={} ):
    """Encodes an interface description block, including the specified options."""
    block_type = 0x00000001
    link_type = pcapng.linktype.LINKTYPE_ETHERNET   # todo how determine?
    reserved = 0
    snaplen = 0                     # 0 => no limit

    for opt_code in opts_dict.keys():
        pcapng.option.assert_if_option(opt_code)
    options_bytes = options_encode(opts_dict)

    pcapng.util.assert_block32_length(options_bytes)
    block_total_len =   (  4 +         # block type
                           4 +         # block total length
                           2 + 2 +     # linktype + reserved
                           4 +         # snaplen
                           len(options_bytes) +
                           4 )         # block total length
    block = ( struct.pack( '=LlHHl', block_type, block_total_len, link_type, reserved, snaplen )
            + options_bytes
            + struct.pack( '=l', block_total_len ))
    return block

def interface_desc_block_decode(block):
    """Decodes a bytes block into an interface description block, returning a dictionary."""
    assert type( block ) == str       #todo is tuple & str ok?
    block_type              = struct.unpack( '=L', block[0:4]   )[0]
    block_total_len         = struct.unpack( '=l', block[4:8]   )[0]
    link_type               = struct.unpack( '=H', block[8:10]  )[0]
    reserved                = struct.unpack( '=H', block[10:12] )[0]
    snaplen                 = struct.unpack( '=l', block[12:16] )[0]

    options_bytes           = block[16:-4]
    options_dict            = options_decode( options_bytes )

    block_total_len_end     = struct.unpack( '=l', block[-4:]   )[0]
    block_data = { 'block_type'             : block_type ,
                   'block_total_len'        : block_total_len ,
                   'link_type'              : link_type ,
                   'reserved'               : reserved ,
                   'snaplen'                : snaplen ,
                   'options_dict'           : options_dict,
                   'block_total_len_end'    : block_total_len_end }
    return block_data


def simple_pkt_block_encode(pkt_data):
    """Encodes a simple packet block."""
    pcapng.util.assert_type_bytes(pkt_data)        #todo is list & tuple & str ok?
    pkt_data            = list(map(ord, pkt_data))
    pkt_data_pad        = pcapng.util.block32_pad_bytes(pkt_data)
    block_type = 0x00000003
    original_pkt_len = len(pkt_data)
    pkt_data_pad_len = len(pkt_data_pad)
    block_total_len = ( 4 +      # block type
                        4 +      # block total length
                        4 +      # original packet length
                        pkt_data_pad_len +
                        4 )      # block total length
    block = ( struct.pack( '=LLL', block_type, block_total_len, original_pkt_len ) +
              pkt_data_pad +
              struct.pack( '=L', block_total_len ))
    return block

def simple_pkt_block_decode(block):
    """Decodes a bytes block into a simple packet block, returning a dictionary."""
    pcapng.util.assert_type_bytes( block )
    block_type          = pcapng.util.first( struct.unpack( '=L', block[0:4]  ))
    block_tot_len       = pcapng.util.first( struct.unpack( '=L', block[4:8]  ))
    original_pkt_len    = pcapng.util.first( struct.unpack( '=L', block[8:12] ))
    pkt_data_pad_len    = pcapng.util.block32_ceil_bytes(original_pkt_len)
    pkt_data            = block[ 12 : (12+original_pkt_len)  ]
    block_tot_len_end   = pcapng.util.first( struct.unpack( '=L', block[ -4:block_tot_len] ))
    block_data =    { 'block_type'          : block_type ,
                      'block_tot_len'         : block_tot_len ,
                      'original_pkt_len'    : original_pkt_len ,
                      'pkt_data_pad_len'    : pkt_data_pad_len ,
                      'pkt_data'            : pkt_data ,
                      'block_tot_len_end'     : block_tot_len_end }
    return block_data




import struct
import time
import pytest
import pcapng.util
from pcapng.util import to_bytes, str_to_bytes

def test_block32_pad_len():
    assert 0 == pcapng.util.block32_ceil_bytes(0)

    assert 4 == pcapng.util.block32_ceil_bytes(1)
    assert 4 == pcapng.util.block32_ceil_bytes(2)
    assert 4 == pcapng.util.block32_ceil_bytes(3)
    assert 4 == pcapng.util.block32_ceil_bytes(4)

    assert 8 == pcapng.util.block32_ceil_bytes(5)
    assert 8 == pcapng.util.block32_ceil_bytes(6)
    assert 8 == pcapng.util.block32_ceil_bytes(7)
    assert 8 == pcapng.util.block32_ceil_bytes(8)

def test_pad_to_len():
    with pytest.raises(AssertionError): pcapng.util.pad_bytes([1, 2, 3, 4], 3)
    with pytest.raises(AssertionError): pcapng.util.pad_bytes('superlong', 3)

    assert to_bytes( 'superlong' + chr(0)*23 ) == pcapng.util.pad_bytes('superlong', 32)
    assert to_bytes( [0, 0, 0, 0] ) == pcapng.util.pad_bytes([          ], 4)
    assert to_bytes( [1, 0, 0, 0] ) == pcapng.util.pad_bytes([1, ], 4)
    assert to_bytes( [1, 2, 0, 0] ) == pcapng.util.pad_bytes([1, 2], 4)
    assert to_bytes( [1, 2, 3, 0] ) == pcapng.util.pad_bytes([1, 2, 3], 4)
    assert to_bytes( [1, 2, 3, 4] ) == pcapng.util.pad_bytes([1, 2, 3, 4], 4)

    assert to_bytes( [9, 9, 9, 9] ) == pcapng.util.pad_bytes([          ], 4, 9)
    assert to_bytes( [1, 9, 9, 9] ) == pcapng.util.pad_bytes([1, ], 4, 9)
    assert to_bytes( [1, 2, 9, 9] ) == pcapng.util.pad_bytes([1, 2], 4, 9)
    assert to_bytes( [1, 2, 3, 9] ) == pcapng.util.pad_bytes([1, 2, 3], 4, 9)
    assert to_bytes( [1, 2, 3, 4] ) == pcapng.util.pad_bytes([1, 2, 3, 4], 4, 9)

def test_pad_to_block32():
    assert to_bytes( [                      ] ) == pcapng.util.block32_pad_bytes([                      ])
    assert to_bytes( [1, 0, 0, 0            ] ) == pcapng.util.block32_pad_bytes([1])
    assert to_bytes( [1, 2, 0, 0            ] ) == pcapng.util.block32_pad_bytes([1, 2])
    assert to_bytes( [1, 2, 3, 0            ] ) == pcapng.util.block32_pad_bytes([1, 2, 3])
    assert to_bytes( [1, 2, 3, 4            ] ) == pcapng.util.block32_pad_bytes([1, 2, 3, 4])
    assert to_bytes( [1, 2, 3, 4, 5, 0, 0, 0] ) == pcapng.util.block32_pad_bytes([1, 2, 3, 4, 5])
    assert to_bytes( [1, 2, 3, 4, 5, 6, 0, 0] ) == pcapng.util.block32_pad_bytes([1, 2, 3, 4, 5, 6])
    assert to_bytes( [1, 2, 3, 4, 5, 6, 7, 0] ) == pcapng.util.block32_pad_bytes([1, 2, 3, 4, 5, 6, 7])
    assert to_bytes( [1, 2, 3, 4, 5, 6, 7, 8] ) == pcapng.util.block32_pad_bytes([1, 2, 3, 4, 5, 6, 7, 8])

    pcapng.util.assert_block32_length([                      ])
    pcapng.util.assert_block32_length([1, 2, 3, 4])
    pcapng.util.assert_block32_length([1, 2, 3, 4, 5, 6, 7, 8])
    with pytest.raises(AssertionError): pcapng.util.assert_block32_length([1])
    with pytest.raises(AssertionError): pcapng.util.assert_block32_length([1, 2])
    with pytest.raises(AssertionError): pcapng.util.assert_block32_length([1, 2, 3])


def test_xxx():
    xx1 = struct.pack(   '!hhl', 1, 2, 3 )  # h='short', l='long'
    xx2 = struct.unpack( '!hhl', xx1 )      # ! => network byte order (big-endian)
    assert xx1 == '\x00\x01\x00\x02\x00\x00\x00\x03'
    assert xx2 == ( 1, 2, 3 )
    assert '\x00\x00\x00\x00\x00\x00\x00\x05' == struct.pack( '!q', 5 )
    assert '\x00\x00\x00\x05'                 == struct.pack( '!l', 5 )
    assert '\x00\x05'                         == struct.pack( '!h', 5 )
    assert 1 == pcapng.util.first( [1,2,3] )

    assert 3 == len( [ 1, 2, 3] )
    assert (3, 140000) == pcapng.util.split_float(3.14)
    assert (3, 141593) == pcapng.util.split_float(3.141592654)

    assert 'abc'             == pcapng.util.ChrList_to_str(['a', 'b', 'c'])

    ts1 = pcapng.util.curr_utc_time_tuple()
    time.sleep(0.1)
    delta = pcapng.util.timetup_subtract( ts1, pcapng.util.curr_utc_time_tuple() )
    assert ((0.09 < delta) and (delta < 0.11))

def test_types():
    pcapng.util.assert_type_str('')
    pcapng.util.assert_type_str('a')
    pcapng.util.assert_type_str('abc')

    pcapng.util.assert_type_list( [] )
    pcapng.util.assert_type_list( [1] )
    pcapng.util.assert_type_list( [1,2,3,] )

    pcapng.util.assert_type_dict( {} )
    pcapng.util.assert_type_dict( {'a':1} )
    pcapng.util.assert_type_dict( {'a':1, 'b':2} )

    with pytest.raises(AssertionError): pcapng.util.assert_type_str( None )
    with pytest.raises(AssertionError): pcapng.util.assert_type_str( [1] )
    with pytest.raises(AssertionError): pcapng.util.assert_type_str( {'a':1} )

    with pytest.raises(AssertionError): pcapng.util.assert_type_list( None )
    with pytest.raises(AssertionError): pcapng.util.assert_type_list( 'a' )
    with pytest.raises(AssertionError): pcapng.util.assert_type_list( {'a':1} )

    with pytest.raises(AssertionError): pcapng.util.assert_type_dict( None )
    with pytest.raises(AssertionError): pcapng.util.assert_type_dict( 'a' )
    with pytest.raises(AssertionError): pcapng.util.assert_type_dict( [1] )


def test_uint8():
    for ub in range(256):
        pcapng.util.assert_uint8(ub)
    with pytest.raises(AssertionError): pcapng.util.assert_uint8(-1)
    with pytest.raises(AssertionError): pcapng.util.assert_uint8(256)

def test_int8():
    for sb in range(-128,127):
        pcapng.util.assert_int8(sb)
    with pytest.raises(AssertionError): pcapng.util.assert_int8(-129)
    with pytest.raises(AssertionError): pcapng.util.assert_int8(128)

def test_bytearray():
    pcapng.util.assert_type_bytearray( bytearray( [1,2,255] ))
    with pytest.raises(AssertionError): pcapng.util.assert_type_bytearray( list( [1,2,255] ) )
    with pytest.raises(AssertionError): pcapng.util.assert_type_bytearray( 'abc' )

def test_to_bytes():
    assert 'abc' == to_bytes( 'abc' )
    assert 'abc' == to_bytes( [97,98,99] )
    if pcapng.util.is_python2():
        assert str( 'abc' ) == to_bytes( 'abc' )
    if pcapng.util.is_python3():
        assert bytes( [97,98,99] ) == to_bytes( [97,98,99] )

def test_str_to_bytes():
    assert to_bytes( [97,98,99] ) == str_to_bytes( 'abc' )
